@@ -8,7 +8,7 @@ from grpclib.client import Channel
 
 from gv_proto import protobuf
 from gv_proto.proto.broadcaster_pb2 import PubRequest, SubRequest
-from gv_proto.proto.archivist_pb2 import DataQuality, Indicators, TravelTimes
+from gv_proto.proto.archivist_pb2 import DataQuality, Indicators, ZonesTravelTime, ZonesTravelTimeRequest
 from gv_proto.proto.geographer_pb2 import Locations, LocationsRequest, Mapping, MappingRequest
 from gv_proto.grpclib.interface_grpc import InterfaceStub
 from gv_utils import enums
@@ -24,11 +24,12 @@ KARRUS_RD = enums.DataTypeId.karrusrd
 DATA_QUALITY = enums.DataTypeId.dataquality
 DATA_POINTS = enums.DataTypeId.datapoints
 MAPPING_ROADS_DATA_POINTS = enums.DataTypeId.mappingroadsdatapoints
-TRAVEL_TIMES = enums.DataTypeId.traveltimes
+ZONES_TRAVEL_TIME = enums.DataTypeId.zonestraveltime
+IMPUTED_PREF = enums.DataTypeId.imputedprefixe
 
 
 class Service:
-    samplings = {KARRUS_RD: 5 * 60, METRO_PME: 1 * 60, TOMTOM_FCD: 1 * 60}
+    samplings = {KARRUS_RD: 1 * 60, METRO_PME: 1 * 60, TOMTOM_FCD: 1 * 60}
 
     def __init__(self, logger, futures=None, callbacks=None):
         if futures is None:
@@ -97,10 +98,12 @@ class Service:
                 encode_func, args = self.__publish_mapping_roads_data_points(data, datatimestamp)
             elif datatype in enums.INDICATORS_DATA_TYPES:
                 encode_func, args = self.__publish_indicators(data)
+            elif IMPUTED_PREF in datatype:
+                encode_func, args = self.__publish_indicators(data)
             elif datatype == DATA_QUALITY:
                 encode_func, args = self.__publish_data_quality(data)
-            elif datatype == TRAVEL_TIMES:
-                encode_func, args = self.__publish_travel_times(data)
+            elif datatype == ZONES_TRAVEL_TIME:
+                encode_func, args = self.__publish_zones_travel_time(data)
             else:
                 raise Exception
 
@@ -138,8 +141,8 @@ class Service:
         return protobuf.encode_data_quality, (data,)
 
     @staticmethod
-    def __publish_travel_times(data):
-        return protobuf.encode_travel_times, (data,)
+    def __publish_zones_travel_time(data):
+        return protobuf.encode_zones_travel_time, (data,)
 
     async def _subscribe(self, datatype, callback):
         async with self.interface.subscribe.open() as stream:
@@ -154,8 +157,8 @@ class Service:
                     pbdata, decode_func = self.__subscribe_indicators()
                 elif datatype == DATA_QUALITY:
                     pbdata, decode_func = self.__subscribe_data_quality()
-                elif datatype == TRAVEL_TIMES:
-                    pbdata, decode_func = self.__subscribe_travel_times()
+                elif datatype == ZONES_TRAVEL_TIME:
+                    pbdata, decode_func = self.__subscribe_zones_travel_time()
                 else:
                     raise Exception
 
@@ -190,8 +193,8 @@ class Service:
         return DataQuality(), protobuf.decode_data_quality
 
     @staticmethod
-    def __subscribe_travel_times():
-        return TravelTimes(), protobuf.decode_travel_times
+    def __subscribe_zones_travel_time():
+        return ZonesTravelTime(), protobuf.decode_zones_travel_time
 
     async def _get_data_points(self, datapointeids=None, datatypeeid=None):
         lr = LocationsRequest()
@@ -205,6 +208,13 @@ class Service:
         lr = LocationsRequest()
         return await protobuf.decode_locations(await self.interface.get_zone_points(lr))
 
+    async def _get_zones_travel_time(self, frompointeid, topointeid, period, fromdatetime, todatetime=None):
+        request = ZonesTravelTimeRequest(fromPoint=frompointeid, toPoint=topointeid, period=period)
+        request.fromdate.FromSeconds(int(fromdatetime.timestamp()))
+        if todatetime is not None:
+            request.todate.FromSeconds(int(todatetime.timestamp()))
+        return await protobuf.decode_zones_travel_time(await self.interface.get_zones_travel_time(request))
+
     async def _get_roads(self, roadeids=None):
         lr = LocationsRequest()
         if roadeids is not None:
